/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.datasource.impl;

import org.springframework.stereotype.Service;

import com.openexchange.office.datasource.access.DataSource;
import com.openexchange.office.datasource.access.DataSourceAccess;
import com.openexchange.office.datasource.access.DataSourceAccessFactory;
import com.openexchange.office.datasource.access.DataSourceException;
import com.openexchange.office.tools.annotation.RegisteredService;
import com.openexchange.office.tools.common.error.ErrorCode;
import com.openexchange.office.tools.common.osgi.context.OsgiBundleContextAndActivator;
import com.openexchange.office.tools.common.osgi.context.OsgiBundleContextAware;
import com.openexchange.tools.session.ServerSession;

@Service
@RegisteredService(registeredClass=DataSourceAccessFactory.class)
public class DataSourceAccessFactoryImpl implements DataSourceAccessFactory, OsgiBundleContextAware {

	private OsgiBundleContextAndActivator bundleCtx;
	
	@Override
	public DataSourceAccess createDataSourceAccess(ServerSession session, String dataSource) throws DataSourceException {
		DataSourceAccess result = null;

		switch (dataSource) {
		case DataSource.DRIVE_GUARD:
		case DataSource.DRIVE: result = new DataSourceAccessDrive(session); break;
		case DataSource.MAIL: result = new DataSourceAccessMail(session); break;
		case DataSource.COMPOSE: result = new DataSourceAccessCompose(session); break;
		case DataSource.TASKS:
		case DataSource.CALENDAR:
		case DataSource.CONTACTS: result = new DataSourceAccessCalendarTaskContact(session, dataSource); break;
		default: throw new DataSourceException(ErrorCode.GENERAL_ARGUMENTS_ERROR);
		}
		bundleCtx.injectDependencies(result);
		return result;
	}

	@Override
	public void setApplicationContext(OsgiBundleContextAndActivator bundleCtx) {
		this.bundleCtx = bundleCtx;
	}
}
